"use strict";
// @ts-check

const NOW = new Date();
const NOW_WITHOUT_TIME = new Date(NOW.getFullYear(), NOW.getMonth(), NOW.getDate());
const HOLIDAY_URL = "https://binarium.de/_media/holiday/holidays.json";
const TRANSLATION_URL = "https://binarium.de/_media/holiday/translations.json";
const rootElementID = "holidays";

let translations;

class Holiday {
    constructor(key, open, date) {
        this.key = key;
        this.open = open;
        this.date = date;
    }
}

class HolidayFormatter {
    constructor(language) {
        this.language = language;
    }

    formatDate(holiday) {
        const optionsDay = { weekday: "long" };
        const options = { year: "numeric", month: "long", day: "2-digit" };

        const dateDay = holiday.date.toLocaleDateString(this.language, optionsDay);

        return (
            holiday.date.toLocaleDateString(this.language, options) +
            " (" +
            dateDay +
            ")"
        );
    }

    formatName(holiday) {
        return this._translate(holiday.key) || "&ndash;"
    }

    formatOpen(holiday) {
        return this._translate(holiday.open); 
    }

    _translate(key) {
        for (let index in translations) {
            if (
                translations[index].key === key &&
                translations[index].lang === this.language
            ) {
                if ("label" in translations[index])
                    return translations[index].label;
                else
                    return undefined;
            }
        }
        return undefined;
    }
}

class HolidayTable {
    create(rootElement, holidays, language) {
        const formatter = new HolidayFormatter(language);

        let isInThisYear = true;
        let table = `<table class='table' id="holidayTable">
        <tr><td><strong>${NOW.getFullYear()}</strong></td><td></td><td></td></tr>`;
        for (const holiday of holidays) {
            if (isInThisYear && holiday.date.getFullYear() > NOW.getFullYear()) {
                isInThisYear = false;
                table += `<tr><td><strong>${NOW.getFullYear() + 1}</strong></td><td></td><td></td></tr>`;
                table += this._tableRow(holiday, formatter);
            }
            else {
                table += this._tableRow(holiday, formatter);
            }
        }
        table += `</table>`;

        document.getElementById(rootElement).innerHTML = table;
    }

    _tableRow(holiday, formatter) {
        const name = "key" in holiday ? formatter.formatName(holiday) : "";
        const date = "date" in holiday ? formatter.formatDate(holiday) : "";
        const open = "open" in holiday ? formatter.formatOpen(holiday) : "";
        return `<tr><td>${name}</td><td>${date}</td><td>${open}</td></tr>`;
    }
}

window.onload = () => {
//    let strPATH = window.location.href;
//    let json1 = loadJsonData(HOLIDAY_URL);
//    let json2 = loadJsonData(TRANSLATION_URL);
    let jsonFiles = [
        loadJsonData(HOLIDAY_URL),
        loadJsonData(TRANSLATION_URL)
    ];

    Promise.all(jsonFiles)
        .then(results => {
            const jsonHolidays = results[0].holidays;
            translations = results[1].translate;

            const holidays = jsonHolidays.map(jsonHoliday =>
                mapJsonHolidayToHoliday(jsonHoliday)
            );

            const compareDates = (a, b) => a.date - b.date;
            holidays.sort(compareDates);

            const holydayTable = new HolidayTable();
            holydayTable.create(rootElementID, holidays, language());
        })
        .catch(results => console.log(results.err, jsonFiles));
};

async function loadJsonData(fileName) {
    const response = await fetch(fileName);
    return await response.json();
}

function mapJsonHolidayToHoliday(jsonHoliday) {
    let date = new Date();
    if ("date" in jsonHoliday) {
        let dayMonth = jsonHoliday.date.split(".");

        date = new Date(NOW.getFullYear(), +dayMonth[1] - 1, +dayMonth[0]);

        if (date < NOW_WITHOUT_TIME) {
            date.setFullYear(date.getFullYear() + 1);
        }
    } else {
        // Assume an algorithm
        let offset = 0;
        if ("offset" in jsonHoliday) offset = +jsonHoliday.offset;
        const functionCall = jsonHoliday.algo + "(" + NOW.getFullYear() + ")";
        date = eval(functionCall);
        date.setDate(date.getDate() + offset);
        if (date < NOW) {
            const functionCall =
                jsonHoliday.algo + "(" + (NOW.getFullYear() + 1) + ")";
            date = eval(functionCall);
            date.setDate(date.getDate() + offset);
        }
    }

    return new Holiday(jsonHoliday.day, jsonHoliday.open, date);
}

function language() {
    return ["de", "ru", "tr", "en"]
        .filter(language => window.location.href.includes("/" + language))
        .concat("de")
        .shift();
}

///////////////////////////////////////////////////////////////////////////////

function eastern(year) {
    var f = Math.floor,
        G = year % 19,
        C = f(year / 100),
        H = (C - f(C / 4) - f((8 * C + 13) / 25) + 19 * G + 15) % 30,
        I = H - f(H / 28) * (1 - f(29 / (H + 1)) * f((21 - G) / 11)),
        J = (year + f(year / 4) + I + 2 - C + f(C / 4)) % 7,
        L = I - J,
        month = 3 + f((L + 40) / 44),
        day = L + 28 - 31 * f(month / 4);

    return new Date(year, month - 1, day);
}

// todo: jeden montag, dienstag geschlossen
